unit MultiControlUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls;

type
  TMultiControl = class(TForm)
    RadioGroup1: TRadioGroup;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    TrackBar1: TTrackBar;
    Button1: TButton;
    Button2: TButton;
    procedure initprogram(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
    procedure RadioButton3Click(Sender: TObject);
    procedure RadioButton1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure idleloop(Sender: TObject; var Done: boolean);
  end;

var
  MultiControl: TMultiControl;

implementation

{$R *.dfm}

const maxmax = 501;

type coefftype = array[1..maxmax, 1..maxmax] of single;
     vectortype = array[1..maxmax] of double; //single;

var ch: char;
    qi, p, e, o, r, temp, d: vectortype;
    inputCoeffs,w: coefftype;
    oldr, oldp, oldq: array[1..maxmax] of integer;
    i,j,maxy, maxx, stage: integer;
    count: longint;
    dt,ko, sumerr, t: double;
    s: string;
    maxmatrix, savepattern: integer;
    start: boolean = false;

{v2 = M*v1 premultiply vector by square matrix}
procedure MV(var M:coefftype; var v1, v2: vectortype);
var i,j: integer;
    result: vectortype;
begin
 for i := 1 to maxmatrix do
 begin
  v2[i] := 0.0;
  for j := 1 to  maxmatrix do
    v2[i] := v2[i] + M[i,j]*v1[j]
 end;
end;

procedure VS(var a,b,c: vectortype); { C = A - B Vector subtract}
var i: integer;
begin
 for i := 1 to  maxmatrix do
 c[i] := a[i] - b[i];
 if c[i] > 1000 then c[i] := 1000
 else
 if c[i] < -1000 then c[i] := -1000;
end;

procedure VA(var a,b,c: vectortype);  { C = A + B vector add}
var i: integer;
begin
 for i := 1 to  maxmatrix do
 c[i] := a[i] + b[i];
end;

{B = k*int(A) vector integrate}
procedure VI(var a,b: vectortype; k: double);
var i: integer;
begin
 for i := 1 to  maxmatrix do
  b[i] := b[i] + k*a[i]*dt;
end;

procedure showmatrix(var m:coefftype; a,b: integer);
var maxj: integer;
begin
  if maxmatrix < 20 then maxj := maxmatrix else maxj := 20;
  for i := 1 to  maxmatrix do
   for j := 1 to  maxj do
   with multicontrol.canvas do
     begin
      textout(a + 4*(i-1), b - 1 + j,
        floattostrf(m[j,i],fffixed,4,1));
     end;
end;

procedure showvector(var v:vectortype; a,b: integer);
var maxi: integer;
begin
  if maxmatrix < 15 then maxi := maxmatrix else maxi := 15;
  for i := 1 to  maxi do
   with multicontrol.canvas do
     begin
      textout(a + 4*(i-1), b +i,
        floattostrf(v[i],fffixed,4,1));
     end;
end;

procedure setmaxmatrix;
begin
 with multicontrol do
 i := trackbar1.position;
 case i of
  1: maxmatrix := 3;
  2: maxmatrix := 5;
  3: maxmatrix := 10;
  4: maxmatrix := 30;
  5: maxmatrix := 50;
  6: maxmatrix := 100;
  7: maxmatrix := 300;
  8: maxmatrix := 500;
 end;

end;

procedure controlloops;
begin
 MV(inputCoeffs,qi, p);    { p = coeffs*qi}
 VS(r,p,e);                { e = r - p}
 VI(e,o,2.0*ko);              { o = o + ko*int(e)}
 MV(w,o,temp);             { temp = w^*o}
 VA(temp, d, qi);          { qi = temp + d}
end;

procedure showvariables;
var i,j,y: integer;
    s: double;
begin
   s := (multicontrol.clientwidth)/(maxmatrix - 1);
 for i := 1 to maxmatrix - 1 do
 with multicontrol,canvas do
 begin
  j := round(s*(i-1)) + 3;
  pen.Color := clBlack;
  y := maxy div 2 - oldr[i];
  if y > 30 then
  ellipse(j-3,y-3,j+3,y + 3);

  y := maxy div 2 - oldp[i];
  if y > 30 then
  ellipse(j-3,y - 3,j+3,y + 3);

  y := maxy div 2 - oldq[i];
  if y > 30 then
  ellipse(j-3,y-3,j+3,y + 3);

  oldr[i] := round(r[i] /3.0);
  oldp[i] := round(p[i] /3.0);
  oldq[i] := round(qi[i] /3.0);

  y := maxy div 2 - oldr[i];
   pen.color := clWhite;
  if y > 30 then
   ellipse(j-3,y - 3,j+3,y + 3);

  y := maxy div 2 - oldp[i];
   pen.color := clRed;
  if y > 30 then
   ellipse(j-3,y-3,j+3,y + 3);

  y := maxy div 2 - oldq[i];
   pen.Color := RGB(255,100,255);
  if y > 30 then
   ellipse(j-3,y-3,j+3,y + 3);

  sumerr := sumerr + abs(e[i]);
 end;

end;

procedure initmatrix;
var i,j: integer;
    sum, rnum: double;
begin
 for i := 1 to  maxmatrix do
  for j := 1 to  maxmatrix do
   inputCoeffs[i,j] := 1.98*(random - 0.5);
 ch := chr(0);
 count := 0;
 sumerr := 100;
 for i := 1 to maxmatrix do
  begin
   o[i] := 1000*(random - 0.5);
   qi[i] := 3000*(random - 0.5);
   case savepattern of
    0: r[i] := maxy/2*sin(i*2*pi/maxmatrix);
    1: r[i] := maxy/2*cos(i*2*pi/maxmatrix);
    2: r[i] := 1500*(random - 0.5);
   end;
 with multicontrol, canvas do
  multicontrol.canvas.TextOut(50,50,'Maxmatrix = ' + inttostr(maxmatrix));
   p[i] := qi[i];
   d[i] := 0.0; // * (random - 0.5);
  end;
  if multicontrol.radiogroup1.itemindex <> -1 then
 multicontrol.radiogroup1.itemindex := savepattern;
 t := 0.0;
 for i := 1 to  maxmatrix do
  for j := 1 to maxmatrix do
   w[j,i] := inputCoeffs[i,j]; {W = transpose(inputCoeffs)}
 ko := 5.0/maxmatrix;
 dt := 0.1;
end;

procedure iterate;
begin
 case stage of
  0: with multicontrol do
      begin
       setmaxmatrix;
       initmatrix;
       maxy := clientheight;
       maxx := clientwidth;
       stage := 1;
      end;
  1:with multicontrol,canvas do
     begin
      brush.color := clBlack;
      fillrect(rect(0,0,clientwidth,clientheight));
      button2.visible := true;
      button2.font.color := clBlack;
      button2.caption := 'Number Points = ' + inttostr(maxmatrix);
      trackbar1.visible := true;
      button1.font.Color := clRED;
      button1.caption := 'START';
      t := 0;
      if start then stage := 2;
     end;
  2: begin
      if not start then stage :=1;
      multicontrol.button1.caption := 'STOP';
      controlloops;
      showvariables;
      t := t + dt*2*pi/30.0*maxmatrix/500.0;
      if t > 2*pi then t := 0;
      r[maxmatrix div 2] := 1000*sin(t + 4*pi/3);
      r[maxmatrix div 4] := -1000*cos(t + 8*pi/3 );
      r[3*maxmatrix div 4] := 1000*cos(t);
  end;
 end;
end;

procedure TMulticontrol.idleloop(Sender: TObject; var Done: boolean);
begin
 done := false;
 iterate;
end;

procedure TMultiControl.RadioButton1Click(Sender: TObject);
begin
 for i := 1 to maxmatrix do
         r[i] := maxy/2*sin(i*2*pi/maxmatrix);
 savepattern := 0;
end;

procedure TMultiControl.RadioButton2Click(Sender: TObject);
begin
 for i := 1 to maxmatrix do
         r[i] := -(maxy - 30)/2*cos(i*2*pi/maxmatrix);
savepattern := 1;
end;

procedure TMultiControl.RadioButton3Click(Sender: TObject);
begin
for i := 1 to maxmatrix do
         r[i] := (maxy - 30)*(random - 0.5);
savepattern := 2;
end;

procedure TMultiControl.initprogram(Sender: TObject);
begin
 randomize;
 maxmatrix := 50;
 initmatrix;
 application.onIdle := idleloop;
 start := false;
 stage := 0;
 end;

procedure TMultiControl.Button1Click(Sender: TObject);
begin
 start := not start;
 if start then button1.caption := 'STOP' else button1.Caption := 'START';
 setmaxmatrix;
 stage := 1;
end;

procedure TMultiControl.TrackBar1Change(Sender: TObject);
var i: integer;
begin
 stage := 0;
 setmaxmatrix;
end;

end.
